import type { Document } from "dbtype/api";
import { Card, CardContent, CardDescription, CardHeader, CardTitle } from "@/components/ui/card";
import TagBadge from "@/components/gallery/TagBadge";
import { useEffect, useRef, useState } from "react";

function clipTagsWhenOverflow(tags: string[], limit: number) {
    let l = 0;
    for (let i = 0; i < tags.length; i++) {
        l += tags[i].length;
        if (l > limit) {
            return tags.slice(0, i);
        }
        l += 1; // for space
    }
    return tags;
}

function LazyImage({ src, alt, className }: { src: string; alt: string; className?: string; }) {
    const ref = useRef<HTMLImageElement>(null);
    const [loaded, setLoaded] = useState(false);

    useEffect(() => {
        if (ref.current) {
            let toggle = false;
            const observer = new IntersectionObserver((entries) => {
                if (entries.some(x => x.isIntersecting)) {
                    setLoaded(true);
                    toggle = !toggle;
                    ref.current?.animate([{ opacity: 0 }, { opacity: 1 }], { duration: 500, easing: "ease-in-out" });
                    observer.disconnect();
                }
                else {
                    if (toggle) {
                        console.log("fade out");
                        ref.current?.animate([{ opacity: 1 }, { opacity: 0 }], { duration: 500, easing: "ease-in-out" });
                    }
                }
            });
            observer.observe(ref.current);
            return () => {
                observer.disconnect();
            };
        }
    }, []);

    return <img
        ref={ref}
        src={loaded ? src : undefined}
        alt={alt}
        className={className}
        loading="lazy"
        />;
}

export function GalleryCard({
    doc: x
}: { doc: Document; }) {
    const ref = useRef<HTMLDivElement>(null);
    const [clipCharCount, setClipCharCount] = useState(200);

    const artists = x.tags.filter(x => x.startsWith("artist:")).map(x => x.replace("artist:", ""));
    const groups = x.tags.filter(x => x.startsWith("group:")).map(x => x.replace("group:", ""));

    const originalTags = x.tags.map(x => x.replace("artist:", "").replace("group:", ""));
    const clippedTags = clipTagsWhenOverflow(originalTags, clipCharCount);

    useEffect(() => {
        const listener = () => {
            if (ref.current) {
                const { width } = ref.current.getBoundingClientRect();
                const charWidth = 8; // rough estimate
                const newClipCharCount = Math.floor(width / charWidth) * 3;
                setClipCharCount(newClipCharCount);
            }
        };
        window.addEventListener("resize", listener);
        return () => {
            window.removeEventListener("resize", listener);
        };
    }, []);

    return <Card className="flex h-[200px]">
        <div className="rounded-xl overflow-hidden h-[200px] w-[142px] flex-none bg-[#272733] flex items-center justify-center">
            <LazyImage src={`/api/doc/${x.id}/comic/thumbnail`}
                alt={x.title}
                className="max-h-full max-w-full object-cover object-center"
                />
        </div>
        <div className="flex-1 flex flex-col">
            <CardHeader className="flex-none">
                <CardTitle>{x.title}</CardTitle>
                <CardDescription>
                    {artists.join(", ")} {groups.length > 0 && "|"} {groups.join(", ")}
                </CardDescription>
            </CardHeader>
            <CardContent className="flex-1" ref={ref}>
                <ul className="flex flex-wrap gap-2 items-baseline content-start">
                    {clippedTags.map(tag => <TagBadge key={tag} tagname={tag} className="" />)}
                    {clippedTags.length < originalTags.length && <TagBadge tagname="..." className="" disabled />}
                </ul>
            </CardContent>
        </div>
    </Card>;
}
